--- original_Fl_Widget_Type.cxx	2008-01-23 11:32:51.000000000 +0000
+++ Fl_Widget_Type.cxx	2008-01-23 11:39:05.000000000 +0000
@@ -1931,6 +1931,31 @@
     else write_c("%s *%s={(%s *)0};\n", t, c, t);
   }
   if (callback() && !is_name(callback())) {
+    // This section writes the callbacks to the cxx file.
+    // Previously, the callbacks are writen outside of declaration blocks (#if ...)
+    // even if the widget containing the callbacks is inside such a block.
+    // The code now looks at the parents of the widget and if it finds #if declaration blocks,
+    // they callbacks are placed inside. Max depth 16 (decl blocks)
+    Fl_Type *db_parents[16] = {0};
+    Fl_Type *this_widget = this;
+    int depth = 0;
+    while ((this_widget->parent) && (depth < 16)){
+      // recurse up the parent tree looking for decl blocks
+      if ((this_widget->parent->is_decl_block()) && !(this_widget->parent->is_class())) {
+        // Don't know why a class returns TRUE for is_decl_block() ???
+        this_widget = db_parents[depth++] = this_widget->parent;
+      }
+      else {
+        // parent found, but not decl block. Look for it's parents
+        this_widget = this_widget->parent;
+      }
+    }
+    // db_parents[] now contains all declaration blocks up to the top level.
+    // Starting at the top level, write the code for the decl blocks
+    while (depth) {
+      db_parents[--depth]->write_code1();
+    }
+    // The #endif's are done later...
     // see if 'o' or 'v' used, to prevent unused argument warnings:
     int use_o = 0;
     int use_v = 0;
@@ -1971,6 +1996,10 @@
         write_c("->user_data()");
       write_c("))->%s_i(o,v);\n}\n", cn);
     }
+     // Write the #endif's that correspond to the decl blocks, in reverse order
+     while (db_parents[depth]) {   // depth will be zero to start (the innermost decl block)
+       db_parents[depth++]->write_code2();
+     }
   }
   if (image) {
     if (image->written != write_number) {
